@@ -8,24 +8,24 @@ Some of the algorithms can be applied in different orders (demosaicing before or
 after linearization, for example), but the reference ordering is as shown below.
 
 Example:
-  raw = rawpipe.downsample(raw, iterations=2)
   raw = rawpipe.linearize(raw, blacklevel=64, whitelevel=1023)
   rgb = rawpipe.wb(rgb, [1.5, 2.0], "RGGB")
-  rgb = rawpipe.demosaic(raw, "RGGB")
+  rgb = rawpipe.demosaic(raw, "RGGB", downsample=True)
+  rgb = rawpipe.downsample(rgb, iterations=1)
   rgb = rawpipe.lsc(rgb, my_vignetting_map)
   rgb = rawpipe.lsc(rgb, my_color_shading_map)
-  rgb = rawpipe.resize(rgb, 400, 300)
   rgb = rawpipe.ccm(rgb, my_3x3_color_matrix)
+  rgb = rawpipe.resize(rgb, 400, 300)
   rgb = rawpipe.gamut(rgb, "ACES")
   rgb = rawpipe.tonemap(rgb, "Reinhard")
   rgb = rawpipe.chroma_denoise(rgb)
-  rgb = pipe.saturate(rgb, lambda x: x ** 0.75)
+  rgb = rawpipe.saturate(rgb, lambda x: x ** 0.75)
   rgb = rawpipe.gamma(rgb, "sRGB")
   rgb = rawpipe.quantize8(rgb)
 
 Example:
   raw = rawpipe.verbose.linearize(raw)
-  raw = rawpipe.silent.demosaic(raw, "RGGB")
+  rgb = rawpipe.silent.demosaic(raw, "RGGB")
 """
 
 from .rawpipe import Algorithms
@@ -39,5 +39,5 @@ _methods = [f for f in dir(Algorithms) if callable(getattr(Algorithms, f)) and n
 for m in _methods:
     globals()[m] = getattr(silent, m)
 
-__version__ = "0.13.0"
+__version__ = "0.14.0"
 __all__ = _methods
