@@ -2,6 +2,7 @@
 A collection of reference ISP algorithms.
 """
 
+import numbers               # built-in library
 import time                  # built-in library
 import warnings              # built-in library
 from typing import Type      # built-in library
@@ -71,29 +72,26 @@ class Algorithms:
 
     def downsample(self, frame: np.ndarray, iterations: int = 1) -> np.ndarray:
         """
-        Downsample the given RGB/Bayer frame by a factor of two in both directions,
-        that is, to a quarter of its original size. This is repeated a given number
-        of iterations. If iterations is zero, the frame is returned untouched. See
-        resize() for arbitrary resizing with proper interpolation in RGB domain.
+        Downsample the given RGB or grayscale frame by a factor of two in both
+        directions, that is, to a quarter of its original size, using a simple
+        2 x 2 box filter. This is repeated for a given number of iterations. If
+        iterations is zero, the frame is returned untouched. See resize() for
+        arbitrary resizing with proper interpolation.
         """
         if iterations >= 1:
             t0 = time.time()
+            dt = frame.dtype
             orgh, orgw = frame.shape[:2]
-            if frame.ndim == 3:  # RGB mode
-                mode = "RGB"
-                frame = frame[::2**iterations, ::2**iterations]
-            if frame.ndim == 2:  # Bayer mode
-                mode = "Bayer"
-                for _ in range(iterations):
-                    ch1, ch2, ch3, ch4 = self.bayer_split(frame)
-                    ch1 = ch1[::2, ::2]
-                    ch2 = ch2[::2, ::2]
-                    ch3 = ch3[::2, ::2]
-                    ch4 = ch4[::2, ::2]
-                    frame = self.bayer_combine(ch1, ch2, ch3, ch4)
+            for _ in range(iterations):
+                ch1, ch2, ch3, ch4 = self.bayer_split(frame)
+                frame = np.stack((ch1, ch2, ch3, ch4))
+                frame = np.mean(frame, axis=0)
+            if issubclass(dt.type, numbers.Integral):
+                frame = np.rint(frame)
+            frame = frame.astype(dt)
             imgh, imgw = frame.shape[:2]
             factor = f"{2**iterations} x {2**iterations}"
-            self._vprint(f"{_elapsed(t0)} - {mode} downsampling [{factor}] from {orgw} x {orgh} to {imgw} x {imgh}")
+            self._vprint(f"{_elapsed(t0)} - downsampling [{factor}] from {orgw} x {orgh} to {imgw} x {imgh}")
         return frame
 
     def resize(self, frame: np.ndarray, target_width: int, target_height: int) -> np.ndarray:
@@ -108,6 +106,7 @@ class Algorithms:
         if (dstw, dsth) != (orgw, orgh):
             dt = frame.dtype
             frame = frame.astype(np.float32) if dt == np.float16 else frame
+            assert frame.dtype in [np.float32, np.float64, np.uint8, np.uint16], f"unsupported dtype: {dt}"
             frame = cv2.resize(frame, (dstw, dsth), cv2.INTER_LANCZOS4)
             frame = frame.astype(dt)
             dsth, dstw = frame.shape[:2]
@@ -374,7 +373,7 @@ class Algorithms:
         converting to HSL. If booster is None, the frame is returned untouched.
 
         Example:
-          img = pipe.saturate(img, lambda x: x ** 0.75)
+          img = rawpipe.saturate(img, lambda x: x ** 0.75)
         """
         if booster is not None:
             t0 = time.time()
@@ -436,6 +435,22 @@ class Algorithms:
             self._vprint(f"{_elapsed(t0)} - applying gamma curve [{realmode}]: range = {input_range} => {self._minmax(frame)}")
         return frame
 
+    def degamma(self, frame: np.ndarray, mode: str = "sRGB") -> np.ndarray:
+        """
+        Apply standard sRGB inverse gamma on the given frame. If mode evaluates to
+        False, the frame is returned untouched.
+        """
+        assert mode in ["sRGB"] or not mode, f"Unrecognized mode '{repr(mode)}'"
+        if mode in ["sRGB"]:
+            t0 = time.time()
+            srgb_lo = frame / 12.92
+            srgb_hi = np.power((frame + 0.055) / 1.055, 2.4)
+            threshold_mask = (frame > 0.04045)
+            input_range = self._minmax(frame)
+            frame = srgb_hi * threshold_mask + srgb_lo * (~threshold_mask)
+            self._vprint(f"{_elapsed(t0)} - applying inverse gamma curve [{mode}]: range = {input_range} => {self._minmax(frame)}")
+        return frame
+
     def quantize(self,
                  frame: np.ndarray,
                  maxval: int = 65535,
