@@ -36,6 +36,8 @@ class RegressionTest(unittest.TestCase):
         raw = alg.gamut(raw, "ACES", p=1.2)
         raw = alg.saturate(raw, lambda x: x ** 0.5)
         raw = rawpipe.verbose.gamma(raw, gamma_mode)
+        raw = rawpipe.verbose.degamma(raw, gamma_mode)
+        raw = rawpipe.verbose.gamma(raw, gamma_mode)
         raw = rawpipe.verbose.quantize(raw, maxval, expected.dtype)
         self.assertEqual(np.count_nonzero(expected - raw), 0)
 
@@ -65,7 +67,7 @@ class RegressionTest(unittest.TestCase):
         self.assertEqual(res.dtype, np.float32)
         np.testing.assert_allclose(org_b * wb[-1], res_b)
 
-    def test_preserve_dtype(self):
+    def test_preserve_dtype_float(self):
         raw = np.random.random((28, 12))
         rgb = np.random.random((28, 12, 3))
         for dt in [np.float16, np.float32, np.float64]:
@@ -92,16 +94,37 @@ class RegressionTest(unittest.TestCase):
             res19 = rawpipe.saturate(rgb_src, lambda v: v ** 0.7)
             res20 = rawpipe.gamma(raw_src)
             res21 = rawpipe.gamma(rgb_src)
-            res22 = rawpipe.ccm(rgb_src, np.random.random((28, 12, 3, 3)))
+            res22 = rawpipe.degamma(rgb_src)
+            res23 = rawpipe.ccm(rgb_src, np.random.random((28, 12, 3, 3)))
             results = [res01, res02, res03, res04, res05, res06,
                        res07, res10, res11, res12, res13, res14,
                        res15, res16, res17, res18, res19, res20,
-                       res21, res22]
-            for res in results:
-                self.assertEqual(res.dtype, dt)
+                       res21, res22, res23]
+            for idx, res in enumerate(results):
+                self.assertEqual(res.dtype, dt, f"results[{idx}] is invalid")
             self.assertEqual(res08.dtype, np.float32)  # linearize()
             self.assertEqual(res09.dtype, np.float32)  # linearize()
 
+    def test_preserve_dtype_int(self):
+        raw = np.random.random((28, 12)) * 100
+        rgb = np.random.random((28, 12, 3)) * 100
+        for dt in [np.int8, np.uint8, np.uint16, np.uint32, np.uint64]:
+            raw_src = raw.astype(dt)
+            rgb_src = rgb.astype(dt)
+            res01 = rawpipe.clip(raw_src, 10, 90)
+            res02 = rawpipe.bayer_combine(*rawpipe.bayer_split(raw_src))
+            res03 = rawpipe.downsample(raw_src, 1)
+            res04 = rawpipe.downsample(rgb_src, 1)
+            results = [res01, res02, res03, res04]
+            for idx, res in enumerate(results):
+                self.assertEqual(res.dtype, dt, f"results[{idx}] is invalid")
+
+    def test_gamma_degamma(self):
+        img = np.random.random((10, 20, 3))
+        np.testing.assert_allclose(rawpipe.degamma(rawpipe.gamma(img)), img, atol=1e-4)
+        self.assertEqual(rawpipe.gamma(img, mode=None).data, img.data)
+        self.assertEqual(rawpipe.degamma(img, mode=None).data, img.data)
+
     def test_errors(self):
         raw = np.fromfile(os.path.join(thisdir, "input.raw"), dtype=np.uint16)
         res = rawpipe.verbose.gamma(raw, None)  # should be no-op
